@@ -30,6 +30,13 @@
 #define MAX_DVD_AUDIO_STREAMS 8
 #define MAX_DVD_SUBPICTURE_STREAMS 32
 
+/* We clamp scr delta with 0 so negative bytes won't be possible */
+#define GSTTIME_TO_BYTES(time) \
+  ((time != -1) ? gst_util_uint64_scale (MAX(0,(gint64) (GSTTIME_TO_MPEGTIME(time) - demux->first_scr)), demux->scr_rate_n, demux->scr_rate_d) : -1)
+#define BYTES_TO_GSTTIME(bytes) ((bytes != -1) ? MPEGTIME_TO_GSTTIME(gst_util_uint64_scale (bytes, demux->scr_rate_d, demux->scr_rate_n)) : -1)
+
+#define ADAPTER_OFFSET_FLUSH(_bytes_) demux->adapter_offset += (_bytes_)
+
 GST_DEBUG_CATEGORY_STATIC (gstflupsdemux_debug);
 #define GST_CAT_DEFAULT (gstflupsdemux_debug)
 
@@ -89,15 +96,16 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 static GstStaticPadTemplate video_template =
-GST_STATIC_PAD_TEMPLATE ("video_%02d",
+GST_STATIC_PAD_TEMPLATE ("video_%02x",
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS ("video/mpeg, "
-        "mpegversion = (int) { 1, 2 }, " "systemstream = (boolean) FALSE")
+        "mpegversion = (int) { 1, 2, 4 }, " "systemstream = (boolean) FALSE;"
+        "video/x-h264")
     );
 
 static GstStaticPadTemplate audio_template =
-    GST_STATIC_PAD_TEMPLATE ("audio_%02d",
+    GST_STATIC_PAD_TEMPLATE ("audio_%02x",
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS ("audio/mpeg, "
@@ -193,17 +201,8 @@ gst_flups_demux_class_init (GstFluPSDemuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_flups_demux_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_flups_demux_get_property);
   gobject_class->finalize = (GObjectFinalizeFunc) gst_flups_demux_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
-      g_param_spec_enum ("sync", "Sync", "The synchronisation method",
-          GST_TYPE_FLUPS_DEMUX_SYNC,
-          GST_FLUPS_DEMUX_SYNC_AUTO, G_PARAM_READWRITE));
-
   gstelement_class->change_state = gst_flups_demux_change_state;
 }
 
@@ -220,7 +219,6 @@ gst_flups_demux_init (GstFluPSDemux * demux)
   demux->streams =
       g_malloc0 (sizeof (GstFluPSStream *) * (GST_FLUPS_DEMUX_MAX_STREAMS));
 
-  demux->cur_sync_method = GST_FLUPS_DEMUX_SYNC_AUTO;
 }
 
 static void
@@ -255,42 +253,6 @@ gst_flups_demux_reset (GstFluPSDemux * demux)
   gst_event_replace (p_ev, NULL);
 }
 
-static void
-gst_flups_demux_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstFluPSDemux *demux;
-
-  demux = GST_FLUPS_DEMUX (object);
-
-  switch (prop_id) {
-    case ARG_SYNC:
-      demux->sync_method = g_value_get_enum (value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_flups_demux_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstFluPSDemux *demux;
-
-  demux = GST_FLUPS_DEMUX (object);
-
-  switch (prop_id) {
-    case ARG_SYNC:
-      g_value_set_enum (value, demux->sync_method);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstFluPSStream *
 gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
 {
@@ -310,6 +272,7 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
   switch (stream_type) {
     case ST_VIDEO_MPEG1:
     case ST_VIDEO_MPEG2:
+    case ST_VIDEO_MPEG4:
     case ST_GST_VIDEO_MPEG1_OR_2: 
     {
       gint mpeg_version = 1;
@@ -318,6 +281,9 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
           demux->is_mpeg2_pack)) {
           mpeg_version = 2;
       }
+      if (stream_type == ST_VIDEO_MPEG4) {
+        mpeg_version = 4;
+      }
 
       template = klass->video_template;
       name = g_strdup_printf ("video_%02x", id);
@@ -338,8 +304,11 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
     case ST_MHEG:
     case ST_DSMCC:
     case ST_AUDIO_AAC:
-    case ST_VIDEO_MPEG4:
+      break;
     case ST_VIDEO_H264:
+      template = klass->video_template;
+      name = g_strdup_printf ("video_%02x", id);
+      caps = gst_caps_new_simple ("video/x-h264", NULL);
       break;
     case ST_PS_AUDIO_AC3:
       template = klass->audio_template;
@@ -373,6 +342,7 @@ gst_flups_demux_create_stream (GstFluPSDemux * demux, gint id, gint stream_type)
   stream = g_new0 (GstFluPSStream, 1);
   stream->id = id;
   stream->discont = TRUE;
+  stream->need_segment = TRUE;
   stream->notlinked = FALSE;
   stream->type = stream_type;
   stream->pad = gst_pad_new_from_template (template, name);
@@ -427,97 +397,42 @@ gst_flups_demux_send_data (GstFluPSDemux * demux, GstFluPSStream * stream,
     goto no_stream;
 
   /* discont */
-  if (stream->discont) {
-    gint64 time, start;
+  if (stream->need_segment) {
+    gint64 time, start, stop;
     GstEvent *newsegment;
+   
+    start = demux->base_time + demux->src_segment.start;
+    if (demux->src_segment.stop != -1) 
+      stop = demux->base_time + demux->src_segment.stop;
+    else
+      stop = -1;
+    time =  start;
 
-    /* When the sync-method is AUTO, SCR/DTS differences 
-     * larger than 1 second cause the syncing mechanism to 
-     * switch to DTS mode */
-    GST_DEBUG_OBJECT (demux,
-        "BEFORE base scr %" G_GUINT64_FORMAT " base_dts %"
-        G_GUINT64_FORMAT " next_dts %" G_GUINT64_FORMAT,
-        demux->base_scr, demux->base_dts, demux->next_dts);
-
-    if (demux->base_dts == G_MAXUINT64) {
-      if (demux->next_dts == G_MAXUINT64) {
-        if (demux->cur_sync_method == GST_FLUPS_DEMUX_SYNC_DTS ||
-            demux->cur_sync_method == GST_FLUPS_DEMUX_SYNC_AUTO) {
-          GST_DEBUG_OBJECT (demux,
-              "Don't have base DTS for this stream. Dropping buf");
-          gst_buffer_unref (buf);
-          return GST_FLOW_OK;
-        }
-      }
-
-      demux->base_dts = demux->next_dts - demux->scr_adjust;
-    }
-
-    GST_DEBUG_OBJECT (demux,
-        "AFTER base scr %" G_GUINT64_FORMAT " base_dts %" G_GUINT64_FORMAT
-        " next_dts %" G_GUINT64_FORMAT,
-        demux->base_scr, demux->base_dts, demux->next_dts);
-
-    if (demux->cur_sync_method == GST_FLUPS_DEMUX_SYNC_AUTO) {
-      gint64 diff;
-      if (demux->base_dts > demux->base_scr)
-        diff = demux->base_dts - demux->base_scr;
-      else
-        diff = demux->base_scr - demux->base_dts;
-
-      if (diff > CLOCK_FREQ) {
-        /* Use DTS as the sync method */
-        if (demux->cur_sync_method != GST_FLUPS_DEMUX_SYNC_DTS) {
-          GST_DEBUG_OBJECT (demux, "Switching stream sync method to DTS");
-          demux->cur_sync_method = GST_FLUPS_DEMUX_SYNC_DTS;
-        }
-      } else {
-        /* Use SCR as the sync method, but for SCR leave the variable 
-         * pointing to AUTO so that if we find bad stuff later, we switch
-         * to DTS */
-        if (demux->cur_sync_method != GST_FLUPS_DEMUX_SYNC_SCR) {
-          GST_DEBUG_OBJECT (demux, "Selecting stream sync method SCR");
-        }
-      }
-    }
-
-    switch (demux->cur_sync_method) {
-      case GST_FLUPS_DEMUX_SYNC_AUTO:
-      case GST_FLUPS_DEMUX_SYNC_SCR:
-        start = MPEGTIME_TO_GSTTIME (demux->base_scr);
-        time = demux->base_time;
-        break;
-      case GST_FLUPS_DEMUX_SYNC_DTS:
-        start = MPEGTIME_TO_GSTTIME (demux->base_dts);
-        time = demux->base_time;
-        break;
-      default:
-        g_warning ("unknown sync mode");
-        start = 0;
-        time = 0;
-        break;
-    }
 #ifdef HAVE_NEWSEG_FULL
     GST_INFO_OBJECT (demux, "sending new segment: rate %g applied_rate %g "
-        "start: %" G_GINT64_FORMAT ", time: %" G_GINT64_FORMAT
-        " to pad %" GST_PTR_FORMAT,
+        "start: %" GST_TIME_FORMAT ", stop: %" GST_TIME_FORMAT
+        ", time: %" GST_TIME_FORMAT " to pad %" GST_PTR_FORMAT,
         demux->sink_segment.rate, demux->sink_segment.applied_rate,
-        start, time, stream->pad);
+        GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
+        GST_TIME_ARGS (time), stream->pad);
 
     newsegment = gst_event_new_new_segment_full (FALSE,
         demux->sink_segment.rate, demux->sink_segment.applied_rate,
-        GST_FORMAT_TIME, start, -1, time);
+        GST_FORMAT_TIME, start, stop, time);
 #else
     GST_INFO_OBJECT (demux, "sending new segment: rate %g "
-        "start: %" G_GINT64_FORMAT ", time: %" G_GINT64_FORMAT
-        " to pad %" GST_PTR_FORMAT,
-        demux->sink_segment.rate, start, time, stream->pad);
+        "start: %" GST_TIME_FORMAT ", stop: %" GST_TIME_FORMAT
+        ", time: %" GST_TIME_FORMAT " to pad %" GST_PTR_FORMAT,
+        demux->sink_segment.rate, GST_TIME_ARGS (start), 
+        GST_TIME_ARGS (stop), GST_TIME_ARGS (time), stream->pad);
 
     newsegment = gst_event_new_new_segment (FALSE,
-        demux->sink_segment.rate, GST_FORMAT_TIME, start, -1, time);
+        demux->sink_segment.rate, GST_FORMAT_TIME, start, stop, time);
 #endif
 
     gst_pad_push_event (stream->pad, newsegment);
+
+    stream->need_segment = FALSE;
   }
 
   /* timestamps */
@@ -561,7 +476,8 @@ no_stream:
 }
 
 static void
-gst_flups_demux_mark_discont (GstFluPSDemux * demux)
+gst_flups_demux_mark_discont (GstFluPSDemux * demux, gboolean discont,
+    gboolean need_segment)
 {
   gint id;
 
@@ -570,8 +486,10 @@ gst_flups_demux_mark_discont (GstFluPSDemux * demux)
     GstFluPSStream *stream = demux->streams[id];
 
     if (stream) {
-      GST_DEBUG_OBJECT (demux, "marking stream as discont");
-      stream->discont = TRUE;
+      stream->discont |= discont;
+      stream->need_segment |= need_segment;
+      GST_DEBUG_OBJECT (demux, "marked stream as discont %d, need_segment %d",
+		      stream->discont, stream->need_segment);
     }
   }
 }
@@ -611,6 +529,9 @@ gst_flups_demux_handle_dvd_event (GstFluPSDemux * demux, GstEvent * event)
     gst_event_replace (p_ev, event);
 
     GST_DEBUG_OBJECT (demux, "Handling language codes event");
+    
+    /* Create a video pad to ensure have it before emit no more pads */
+    temp = gst_flups_demux_get_stream (demux, 0xe0, ST_VIDEO_MPEG2);
 
     /* Read out the languages for audio streams and request each one that 
      * is present */
@@ -672,6 +593,7 @@ gst_flups_demux_handle_dvd_event (GstFluPSDemux * demux, GstEvent * event)
         "signalling no-more-pads");
 
     gst_element_no_more_pads (GST_ELEMENT (demux));
+    demux->need_no_more_pads = FALSE;
   }
 
   gst_event_unref (event);
@@ -694,8 +616,9 @@ gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
       gst_flups_demux_send_event (demux, event);
 
       gst_segment_init (&demux->sink_segment, GST_FORMAT_UNDEFINED);
-      gst_flups_demux_mark_discont (demux);
       gst_adapter_clear (demux->adapter);
+      gst_adapter_clear (demux->rev_adapter);
+      demux->adapter_offset = G_MAXUINT64;
       gst_pes_filter_drain (&demux->filter);
       demux->current_scr = G_MAXUINT64;
       demux->bytes_since_scr = 0;
@@ -715,21 +638,38 @@ gst_flups_demux_sink_event (GstPad * pad, GstEvent * event)
           &start, &stop, &time);
       gst_segment_set_newsegment_full (&demux->sink_segment, update, rate,
           arate, format, start, stop, time);
+
+      if (format == GST_FORMAT_BYTES && demux->scr_rate_n != G_MAXUINT64 
+        && demux->scr_rate_d != G_MAXUINT64 ) {
+
+        gst_segment_set_newsegment_full (&demux->src_segment, update, rate,
+            arate, GST_FORMAT_TIME, BYTES_TO_GSTTIME(start), 
+            BYTES_TO_GSTTIME(stop), BYTES_TO_GSTTIME(time));
+      } 
+      
    }
 #else
       gst_event_parse_new_segment (event, &update, &rate, &format,
           &start, &stop, &time);
       gst_segment_set_newsegment (&demux->sink_segment, update, rate,
           format, start, stop, time);
+      if (format == GST_FORMAT_BYTES && demux->scr_rate_n != G_MAXUINT64 
+        && demux->scr_rate_d != G_MAXUINT64 ) {
+
+        gst_segment_set_newsegment (&demux->src_segment, update, rate,
+            GST_FORMAT_TIME, BYTES_TO_GSTTIME(start), BYTES_TO_GSTTIME(stop),
+            BYTES_TO_GSTTIME(time));
+      }
 #endif
 
-      GST_INFO_OBJECT (demux,
-          "Received new segment, format %d, start %" G_GINT64_FORMAT
-          " and time %" G_GINT64_FORMAT, format, start, time);
+      GST_INFO_OBJECT (demux, "received new segment: rate %g "
+        "format %d, start: %" G_GINT64_FORMAT ", stop: %" G_GINT64_FORMAT
+        ", time: %" G_GINT64_FORMAT, rate, format, start, stop, time);
+
+      /* we need to emit a new segment */
+      gst_flups_demux_mark_discont (demux, TRUE, TRUE);
+      gst_event_unref (event);
 
-      demux->current_scr = G_MAXUINT64;
-      demux->bytes_since_scr = 0;
-      demux->scr_adjust = 0;
       break;
     }
     case GST_EVENT_EOS:
@@ -774,19 +714,24 @@ gst_flups_demux_src_event (GstPad * pad, GstEvent * event)
       gdouble rate;
       GstFormat format;
       GstSeekFlags flags;
-      GstSeekType cur_type, stop_type;
-      gint64 cur, stop;
-      gint64 bcur, bstop;
+      GstSeekType start_type, stop_type;
+      gint64 start, stop;
+      gint64 bstart, bstop;
       GstEvent *bevent;
+      gboolean update = FALSE;
 
-      gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
+      gst_event_parse_seek (event, &rate, &format, &flags, &start_type, &start,
           &stop_type, &stop);
 
+      GST_DEBUG_OBJECT (demux, "seek event, rate: %f start: %" GST_TIME_FORMAT
+        " stop: %" GST_TIME_FORMAT, rate, GST_TIME_ARGS (start),
+        GST_TIME_ARGS (stop));
+
       if (format == GST_FORMAT_BYTES) {
         GST_DEBUG_OBJECT (demux, "seek not supported on format %d", format);
         goto not_supported;
       }
-
+     
       GST_DEBUG_OBJECT (demux, "seek - trying directly upstream first");
 
       /* first try original format seek */
@@ -800,20 +745,21 @@ gst_flups_demux_src_event (GstPad * pad, GstEvent * event)
         goto not_supported;
       }
 
-      /* else convert to byte based seek. We need a muxrate for that. */
-      if (demux->mux_rate == -1) {
-        GST_DEBUG_OBJECT (demux, "seek not possible, no mux_rate");
+      /* We need to convert to byte based seek and we need a scr_rate for that. */
+      if (demux->scr_rate_n == G_MAXUINT64 || demux->scr_rate_d == G_MAXUINT64 ) {
+        GST_DEBUG_OBJECT (demux, "seek not possible, no scr_rate");
         goto not_supported;
       }
 
-      bcur = cur * demux->mux_rate / GST_SECOND;
-      if (stop != -1)
-        bstop = stop * demux->mux_rate / GST_SECOND;
-      else
-        bstop = -1;
+      GST_DEBUG_OBJECT (demux, "try with scr_rate interpolation");
 
-      bevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags, cur_type,
-          bcur, stop_type, bstop);
+      bstart = GSTTIME_TO_BYTES(start);
+      bstop = GSTTIME_TO_BYTES(stop);
+
+      GST_DEBUG_OBJECT (demux, "in bytes bstart %" G_GINT64_FORMAT " bstop %" 
+            G_GINT64_FORMAT, bstart, bstop);
+      bevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags, start_type,
+          bstart, stop_type, bstop);
 
       res = gst_pad_push_event (demux->sinkpad, bevent);
 
@@ -863,12 +809,16 @@ gst_flups_demux_src_query (GstPad * pad, GstQuery * query)
             format);
         goto not_supported;
       }
-      if (demux->current_scr == G_MAXUINT64 || demux->base_scr == G_MAXUINT64) {
+      if (demux->current_scr == G_MAXUINT64 || demux->first_scr == G_MAXUINT64) {
         GST_DEBUG_OBJECT (demux, "position not possible, no current_scr");
         goto not_supported;
       }
 
-      position = MPEGTIME_TO_GSTTIME (demux->current_scr - demux->base_scr);
+      position = demux->base_time + 
+        MPEGTIME_TO_GSTTIME (demux->current_scr - demux->first_scr);
+
+      GST_LOG_OBJECT (demux, "Position at GStreamer Time:%" GST_TIME_FORMAT,
+          GST_TIME_ARGS (position));
 
       gst_query_set_position (query, format, position);
       res = TRUE;
@@ -927,7 +877,7 @@ gst_flups_demux_src_query (GstPad * pad, GstQuery * query)
 
       gst_query_parse_duration (query, &format, &duration);
 
-      duration = duration * GST_SECOND / demux->mux_rate;
+      duration = BYTES_TO_GSTTIME(duration);
 
       gst_query_set_duration (query, GST_FORMAT_TIME, duration);
       res = TRUE;
@@ -1025,12 +975,16 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
       goto lost_sync;
 
     scr_ext = (scr2 & 0x03fe0000) >> 17;
+    /* We keep the offset of this scr */
+    demux->last_scr_offset = demux->adapter_offset + 12;
+
     GST_DEBUG_OBJECT (demux, "SCR: 0x%08x SCRE: 0x%08x", scr, scr_ext);
 
     if (scr_ext) {
       scr = (scr * 300 + scr_ext % 300) / 300;
     }
-
+    /* SCR has been converted into units of 90Khz ticks to make it comparable
+       to DTS/PTS, that also implies 1 tick rounding error */
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
     next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
@@ -1064,6 +1018,9 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     scr |= ((guint64) scr1 & 0x000000ff) << 7;
     scr |= ((guint64) scr2 & 0xfe000000) >> 25;
 
+    /* We keep the offset of this scr */
+    demux->last_scr_offset = demux->adapter_offset + 8;
+
     /* marker:1==1 ! mux_rate:22 ! marker:1==1 */
     new_rate = (scr2 & 0x007ffffe) >> 1;
 
@@ -1071,43 +1028,46 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
   }
   new_rate *= MPEG_MUX_RATE_MULT;
 
+  /* scr adjusted is the new scr found + the colected adjustment */
   scr_adjusted = scr + demux->scr_adjust;
+
   GST_LOG_OBJECT (demux,
       "SCR: %" G_GINT64_FORMAT " (%" G_GINT64_FORMAT "), mux_rate %"
-      G_GINT64_FORMAT, scr, scr_adjusted, new_rate);
+      G_GINT64_FORMAT ", GStreamer Time:%" GST_TIME_FORMAT,
+      scr, scr_adjusted, new_rate, 
+      GST_TIME_ARGS (MPEGTIME_TO_GSTTIME ((guint64)scr-demux->first_scr)));
 
-  if (demux->current_scr == G_MAXUINT64) {
-    /* take new base_scr */
-    demux->base_scr = scr;
-    /* we don't know base DTS yet */
-    demux->base_dts = G_MAXUINT64;
+  /* keep the first src in order to calculate delta time */
+  if (demux->first_scr == G_MAXUINT64) {
+    demux->first_scr = scr;
+    demux->first_scr_offset = demux->last_scr_offset;
+    demux->base_time = MPEGTIME_TO_GSTTIME(demux->first_scr);
+    /* at begin consider the new_rate as the scr rate, bytes/clock ticks */
+    demux->scr_rate_n = new_rate;
+    demux->scr_rate_d = CLOCK_FREQ; 
+  }
+  else if (demux->first_scr_offset != demux->last_scr_offset) {
+    /* estimate byte rate related to the SCR */
+    demux->scr_rate_n = demux->last_scr_offset - demux->first_scr_offset;
+    demux->scr_rate_d = scr - demux->first_scr;
+  }
 
-    /* By default, we don't know our base_time */
-    demux->base_time = 0;
+  GST_DEBUG_OBJECT (demux, "%s mode scr: %" G_GUINT64_FORMAT " at %" 
+      G_GUINT64_FORMAT ", first scr: %" G_GUINT64_FORMAT
+      " at %" G_GUINT64_FORMAT ", scr rate: %" G_GUINT64_FORMAT
+      "/%" G_GUINT64_FORMAT "(%f)", 
+      ((demux->sink_segment.rate >= 0.0) ? "forward" : "backward"),
+      scr, demux->last_scr_offset,
+      demux->first_scr, demux->first_scr_offset, 
+      demux->scr_rate_n, demux->scr_rate_d,
+      (float) demux->scr_rate_n / demux->scr_rate_d);
 
-    if (demux->sink_segment.time != -1) {
-      if (demux->sink_segment.format == GST_FORMAT_BYTES && new_rate > 0) {
-        /* this is the time in the stream */
-        demux->base_time = demux->sink_segment.time * GST_SECOND / new_rate;
-      } else if (demux->sink_segment.format == GST_FORMAT_TIME) {
-        demux->base_time = demux->sink_segment.time;
-      }
-    }
-
-    /* mark discont on all streams */
-    gst_flups_demux_mark_discont (demux);
-
-    /* Reset the sync method, so that it will switch again
-     * if it's AUTO */
-    demux->cur_sync_method = demux->sync_method;
-
-    GST_DEBUG_OBJECT (demux, "new base SCR: %" G_GUINT64_FORMAT
-        " base_time: %" GST_TIME_FORMAT, demux->base_scr,
-        GST_TIME_ARGS (demux->base_time));
-  } else {
+  /* adjustment of the SCR */
+  if (demux->current_scr != G_MAXUINT64) {
     gint64 diff;
-    guint64 old_scr, old_mux_rate, bss;
+    guint64 old_scr, old_mux_rate, bss, adjust;
 
+    /* keep SCR of the previous packet */
     old_scr = demux->current_scr;
     old_mux_rate = demux->mux_rate;
 
@@ -1117,18 +1077,28 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     bss = MAX (0, (gint) (demux->bytes_since_scr -
         gst_adapter_available (demux->adapter)));
 
-    demux->next_scr = old_scr + (bss * CLOCK_FREQ) / old_mux_rate;
+    /* estimate the new SCR using the previous one according the notes
+       on point 2.5.2.2 of the ISO/IEC 13818-1 document */ 
+    adjust = (bss * CLOCK_FREQ) / old_mux_rate;
+    if (demux->sink_segment.rate >= 0.0) 
+      demux->next_scr = old_scr + adjust;
+    else 
+      demux->next_scr = old_scr - adjust;
 
     GST_DEBUG_OBJECT (demux,
         "bss: %" G_GUINT64_FORMAT ", next_scr: %" G_GUINT64_FORMAT
         ", old_scr: %" G_GUINT64_FORMAT ", scr: %" G_GUINT64_FORMAT,
         bss, demux->next_scr, old_scr, scr_adjusted);
 
-    if (demux->current_scr > scr_adjusted)
-      diff = demux->current_scr - scr_adjusted;
+    /* calculate the absolute deference between the last scr and
+       the new one */
+    if (old_scr > scr_adjusted)
+      diff = old_scr - scr_adjusted;
     else
-      diff = scr_adjusted - demux->current_scr;
+      diff = scr_adjusted - old_scr;
 
+    /* if the difference is more than 1 second we need to reconfigure
+       adjustment */
     if (diff > CLOCK_FREQ) {
       demux->scr_adjust = demux->next_scr - scr;
       GST_DEBUG_OBJECT (demux, "discont found, diff: %" G_GINT64_FORMAT
@@ -1139,6 +1109,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
     }
   }
 
+  /* update the current_scr and rate members */
   demux->mux_rate = new_rate;
   demux->current_scr = scr_adjusted;
 
@@ -1147,7 +1118,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
   demux->bytes_since_scr = gst_adapter_available (demux->adapter);
 
   gst_adapter_flush (demux->adapter, length);
-
+  ADAPTER_OFFSET_FLUSH (length);
   return GST_FLOW_OK;
 
 lost_sync:
@@ -1299,7 +1270,7 @@ gst_flups_demux_parse_sys_head (GstFluPSDemux * demux)
   }
 
   gst_adapter_flush (demux->adapter, length);
-
+  ADAPTER_OFFSET_FLUSH (length);
   return GST_FLOW_OK;
 
   /* ERRORS */
@@ -1326,6 +1297,101 @@ need_more_data:
   }
 }
 
+static GstFlowReturn
+gst_flups_demux_parse_psm (GstFluPSDemux * demux)
+{
+  guint16 length = 0, info_length = 0, es_map_length = 0;
+  guint8 psm_version = 0;
+  const guint8 *data, *es_map_base;
+  gboolean applicable;
+
+  /* start code + length */
+  if (!(data = gst_adapter_peek (demux->adapter, 6)))
+    goto need_more_data;
+
+  /* skip start code */
+  data += 4;
+
+  length = GST_READ_UINT16_BE (data);
+  GST_DEBUG_OBJECT (demux, "length %u", length);
+  
+  if (G_UNLIKELY (length > 0x3FA))
+    goto psm_len_error;
+
+  length += 6;
+
+  if (!(data = gst_adapter_peek (demux->adapter, length)))
+    goto need_more_data;
+
+  /* skip start code and length */
+  data += 6;
+  
+  /* Read PSM applicable bit together with version */
+  psm_version = GST_READ_UINT8 (data);
+  applicable = (psm_version & 0x80) >> 7;
+  psm_version &= 0x1F;
+  GST_DEBUG_OBJECT (demux, "PSM version %u (applicable now %u)", psm_version,
+      applicable);
+  
+  /* Jump over version and marker bit */
+  data += 2;
+  
+  /* Read PS info length */
+  info_length = GST_READ_UINT16_BE (data);
+  /* Cap it to PSM length - needed bytes for ES map length and CRC */
+  info_length = MIN (length - 16, info_length);
+  GST_DEBUG_OBJECT (demux, "PS info length %u bytes", info_length);
+  
+  /* Jump over that section */
+  data += (2 + info_length);
+  
+  /* Read ES map length */
+  es_map_length = GST_READ_UINT16_BE (data);
+  /* Cap it to PSM remaining length -  CRC */
+  es_map_length = MIN (length - (16 + info_length) , es_map_length);
+  GST_DEBUG_OBJECT (demux, "ES map length %u bytes", es_map_length);
+  
+  /* Jump over the size */
+  data += 2;
+  
+  /* Now read the ES map */
+  es_map_base = data; 
+  while (es_map_base + 4 <= data + es_map_length) {
+    guint8 stream_type = 0, stream_id = 0;
+    guint16 stream_info_length = 0;
+    
+    stream_type = GST_READ_UINT8 (es_map_base);
+    es_map_base++;
+    stream_id = GST_READ_UINT8 (es_map_base);
+    es_map_base++;
+    stream_info_length = GST_READ_UINT16_BE (es_map_base);
+    es_map_base += 2;
+    /* Cap stream_info_length */
+    stream_info_length = MIN (data + es_map_length - es_map_base,
+        stream_info_length);
+    
+    GST_DEBUG_OBJECT (demux, "Stream type %02X with id %02X and %u bytes info",
+        stream_type, stream_id, stream_info_length);
+    demux->psm[stream_id] = stream_type;
+    es_map_base += stream_info_length;
+  }
+  
+  gst_adapter_flush (demux->adapter, length);
+  ADAPTER_OFFSET_FLUSH (length); 
+  return GST_FLOW_OK;
+  
+psm_len_error:
+  {
+    GST_DEBUG_OBJECT (demux, "error in PSM length");
+    return GST_FLOW_LOST_SYNC;
+  }
+need_more_data:
+  {
+    GST_DEBUG_OBJECT (demux, "need more data");
+    return GST_FLOW_NEED_MORE_DATA;
+  }
+}
+
 static void
 gst_flups_demux_resync_cb (GstPESFilter * filter, GstFluPSDemux * demux)
 {
@@ -1412,6 +1478,13 @@ gst_flups_demux_data_cb (GstPESFilter * filter, gboolean first,
     goto done;
   }
 
+  /* After 2 seconds of bitstream emit no more pads */
+  if (demux->need_no_more_pads && (demux->current_scr - demux->first_scr) > 2 * CLOCK_FREQ) {
+    GST_DEBUG_OBJECT (demux, "no more pads, notifying");
+    gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
+    demux->need_no_more_pads=FALSE;
+  }
+
   /* If the stream is not-linked, don't bother creating a sub-buffer
    * to send to it, unless we're processing a discont (which resets
    * the not-linked status and tries again */
@@ -1439,13 +1512,13 @@ done:
 unknown_stream_type:
   {
     GST_DEBUG_OBJECT (demux, "unknown stream type %02x", id);
-    ret = GST_FLOW_ERROR;
+    ret = GST_FLOW_OK;
     goto done;
   }
 }
 
 static gboolean
-gst_flups_demux_resync (GstFluPSDemux * demux)
+gst_flups_demux_resync (GstFluPSDemux * demux, gboolean save)
 {
   const guint8 *data;
   gint avail;
@@ -1457,7 +1530,8 @@ gst_flups_demux_resync (GstFluPSDemux * demux)
   if (avail < 4)
     goto need_data;
 
-  data = gst_adapter_peek (demux->adapter, avail);
+  /* Common case, read 4 bytes an check it */
+  data = gst_adapter_peek (demux->adapter, 4);
 
   /* read currect code */
   code = GST_READ_UINT32_BE (data);
@@ -1469,17 +1543,41 @@ gst_flups_demux_resync (GstFluPSDemux * demux)
     return TRUE;
   }
 
-  /* Otherwise, we are starting at byte 4 */
+  /* Otherwise, we are starting at byte 4 and we need to search 
+     the sync code in all available data in the adapter */
   offset = 4;
   if (offset >= avail)
-    return FALSE;               /* Not enough data to find sync */
+    goto need_data;               /* Not enough data to find sync */
+
+  data = gst_adapter_peek (demux->adapter, avail);
 
   do {
     code = (code << 8) | data[offset++];
     found = (code & 0xffffff00) == 0x100L;
   } while (offset < avail && !found);
 
-  gst_adapter_flush (demux->adapter, offset - 4);
+  if (!save || demux->sink_segment.rate >= 0.0) {
+    GST_LOG_OBJECT (demux, "flushing %d bytes", offset - 4);
+    /* forward playback, we can discard and flush the skipped bytes */
+    gst_adapter_flush (demux->adapter, offset - 4);
+    ADAPTER_OFFSET_FLUSH (offset - 4);
+  }
+  else {
+    if (found) {
+      GST_LOG_OBJECT (demux, "reverse saving %d bytes", offset - 4);
+      /* reverse playback, we keep the flushed bytes and we will append them to
+       * the next buffer in the chain function, which is the previous buffer in
+       * the stream. */
+      gst_adapter_push (demux->rev_adapter,
+  	  gst_adapter_take_buffer (demux->adapter, offset - 4));
+    }
+    else {
+      GST_LOG_OBJECT (demux, "reverse saving %d bytes", avail);
+      /* nothing found, keep all bytes */
+      gst_adapter_push (demux->rev_adapter,
+  	  gst_adapter_take_buffer (demux->adapter, avail));
+    }
+  }
 
   if (found) {
     GST_LOG_OBJECT (demux, "Found resync code %08x after %d bytes",
@@ -1492,7 +1590,10 @@ gst_flups_demux_resync (GstFluPSDemux * demux)
   return found;
 
 need_data:
-  return FALSE;
+  {
+    GST_LOG_OBJECT (demux, "we need more data for resync %d", avail);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -1508,24 +1609,46 @@ gst_flups_demux_chain (GstPad * pad, GstBuffer * buffer)
   GstFluPSDemux *demux = GST_FLUPS_DEMUX (gst_pad_get_parent (pad));
   GstFlowReturn ret = GST_FLOW_OK;
   guint32 avail;
+  gboolean save, discont;
 
-  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
-    GST_LOG_OBJECT (demux, "Received buffer with discont flag");
+  discont = GST_BUFFER_IS_DISCONT (buffer);
+  
+  if (discont) {
+    GST_LOG_OBJECT (demux, "Received buffer with discont flag and"
+        " offset %" G_GUINT64_FORMAT, GST_BUFFER_OFFSET(buffer));
 
     gst_pes_filter_drain (&demux->filter);
+    gst_flups_demux_mark_discont (demux, TRUE, FALSE);
 
     /* mark discont on all streams */
-    gst_flups_demux_mark_discont (demux);
-
-    demux->current_scr = G_MAXUINT64;
-    demux->bytes_since_scr = 0;
+    if (demux->sink_segment.rate >= 0.0) {
+      demux->current_scr = G_MAXUINT64;
+      demux->bytes_since_scr = 0;
+    }
   }
+  else {
+    GST_LOG_OBJECT (demux, "Received buffer with offset %" G_GUINT64_FORMAT,
+        GST_BUFFER_OFFSET(buffer));
+  }
+
+  /* We keep the offset to interpolate SCR */
+  demux->adapter_offset = GST_BUFFER_OFFSET(buffer);
 
   gst_adapter_push (demux->adapter, buffer);
   demux->bytes_since_scr += GST_BUFFER_SIZE (buffer);
 
+  avail = gst_adapter_available (demux->rev_adapter);
+  if (avail > 0) {
+    GST_LOG_OBJECT (demux, "appending %u saved bytes", avail);
+    /* if we have a previous reverse chunk, append this now */
+    /* FIXME this code assumes we receive discont buffers all thei
+     * time */
+    gst_adapter_push (demux->adapter,
+	gst_adapter_take_buffer (demux->rev_adapter, avail));
+  }
+
   avail = gst_adapter_available (demux->adapter);
-  GST_LOG_OBJECT (demux, "avail now: %d", avail);
+  GST_LOG_OBJECT (demux, "avail now: %d, state %d", avail, demux->filter.state);
 
   switch (demux->filter.state) {
     case STATE_DATA_SKIP:
@@ -1554,8 +1677,12 @@ gst_flups_demux_chain (GstPad * pad, GstBuffer * buffer)
       goto done;
   }
 
-  /* align adapter data to sync boundary */
-  while (gst_flups_demux_resync (demux)) {
+  /* align adapter data to sync boundary, we keep the data up to the next sync
+   * point. */
+  save = TRUE;
+  while (gst_flups_demux_resync (demux, save)) { 
+    gboolean ps_sync = TRUE;
+
     /* now switch on last synced byte */
     switch (demux->last_sync_code) {
       case ID_PS_PACK_START_CODE:
@@ -1567,23 +1694,40 @@ gst_flups_demux_chain (GstPad * pad, GstBuffer * buffer)
       case ID_PS_END_CODE:
         ret = GST_FLOW_OK;
         goto done;
+      case ID_PS_PROGRAM_STREAM_MAP:
+        ret = gst_flups_demux_parse_psm (demux);
+        break;
       default:
         if (gst_flups_demux_is_pes_sync (demux->last_sync_code)) {
           ret = gst_pes_filter_process (&demux->filter);
         } else {
-          GST_DEBUG_OBJECT (demux, "non PES sync found, continuing");
+          GST_DEBUG_OBJECT (demux, "sync_code=%08x, non PES sync found"
+              ", continuing", demux->last_sync_code);
+          ps_sync = FALSE;
           ret = GST_FLOW_LOST_SYNC;
         }
         break;
     }
+    /* if we found a ps sync, we stop saving the data, any non-ps sync gets
+     * saved up to the next ps sync. */
+    if (ps_sync)
+      save = FALSE;
+
     switch (ret) {
       case GST_FLOW_NEED_MORE_DATA:
         GST_DEBUG_OBJECT (demux, "need more data");
         ret = GST_FLOW_OK;
         goto done;
       case GST_FLOW_LOST_SYNC:
-        GST_DEBUG_OBJECT (demux, "flushing 3 bytes");
-        gst_adapter_flush (demux->adapter, 3);
+        if (!save || demux->sink_segment.rate >= 0.0) {
+          GST_DEBUG_OBJECT (demux, "flushing 3 bytes");
+          gst_adapter_flush (demux->adapter, 3);
+          ADAPTER_OFFSET_FLUSH (3);
+	}
+	else {
+          GST_DEBUG_OBJECT (demux, "saving 3 bytes");
+          gst_adapter_push (demux->rev_adapter, gst_adapter_take_buffer (demux->adapter, 3));
+	}
         ret = GST_FLOW_OK;
         break;
       default:
@@ -1605,18 +1749,29 @@ gst_flups_demux_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       demux->adapter = gst_adapter_new ();
-      gst_pes_filter_init (&demux->filter, demux->adapter);
+      demux->rev_adapter = gst_adapter_new ();
+      demux->adapter_offset = G_MAXUINT64;
+      gst_pes_filter_init (&demux->filter, demux->adapter, &demux->adapter_offset);
       gst_pes_filter_set_callbacks (&demux->filter,
           (GstPESFilterData) gst_flups_demux_data_cb,
           (GstPESFilterResync) gst_flups_demux_resync_cb, demux);
       demux->filter.gather_pes = TRUE;
+      demux->first_scr = G_MAXUINT64;
+      demux->current_scr = G_MAXUINT64;
+      demux->base_time = G_MAXUINT64;
+      demux->scr_rate_n = G_MAXUINT64;
+      demux->scr_rate_d = G_MAXUINT64;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       demux->current_scr = G_MAXUINT64;
-      demux->base_dts = G_MAXUINT64;
       demux->mux_rate = G_MAXUINT64;
       demux->next_pts = G_MAXUINT64;
       demux->next_dts = G_MAXUINT64;
+      demux->first_scr = G_MAXUINT64;
+      demux->base_time = G_MAXUINT64;
+      demux->scr_rate_n = G_MAXUINT64;
+      demux->scr_rate_d = G_MAXUINT64;
+      demux->need_no_more_pads = TRUE;
 
       gst_flups_demux_reset_psm (demux);
       gst_segment_init (&demux->sink_segment, GST_FORMAT_UNDEFINED);
@@ -1636,6 +1791,8 @@ gst_flups_demux_change_state (GstElement * element, GstStateChange transition)
       gst_pes_filter_uninit (&demux->filter);
       g_object_unref (demux->adapter);
       demux->adapter = NULL;
+      g_object_unref (demux->rev_adapter);
+      demux->rev_adapter = NULL;
       break;
     default:
       break;
