@@ -56,55 +56,59 @@ typedef enum {
 
 /* Information associated with a single FluPS stream. */
 struct _GstFluPSStream {
-  GstPad	*pad;
+  GstPad         * pad;
 
-  gint		 id;
-  gint		 type;
-  gint		 size_bound;
+  gint           id;
+  gint           type;
+  gint           size_bound;
 
   gboolean       discont;
   gboolean       notlinked;
+  gboolean       need_segment;
 };
 
 struct _GstFluPSDemux {
-  GstElement	  parent;
+  GstElement     parent;
 
-  GstPad	 *sinkpad;
+  GstPad         * sinkpad;
 
-  GstAdapter	 *adapter;
-  guint32	  last_sync_code;
-  GstPESFilter    filter;
+  GstAdapter     * adapter;
+  GstAdapter     * rev_adapter;
+  guint64        adapter_offset;
+  guint32        last_sync_code;
+  GstPESFilter   filter;
 
-  gint64	  mux_rate;
-  guint64	  base_scr;
-  guint64	  base_dts;
-  guint64	  base_time;
-  guint64	  current_scr;
-  guint64	  next_scr;
-  guint64	  bytes_since_scr;
-  gint64	  scr_adjust;
+  gint64         mux_rate;
+  guint64	     first_scr;
+  guint64	     first_dts;
+  guint64	     base_time;
+  guint64        current_scr;
+  guint64        next_scr;
+  guint64        bytes_since_scr;
+  gint64         scr_adjust;
+  guint64        scr_rate_n;
+  guint64        scr_rate_d;
+  guint64        first_scr_offset;
+  guint64        last_scr_offset;
 
-  gint16	  psm[GST_FLUPS_DEMUX_MAX_PSM];
+  gint16         psm[GST_FLUPS_DEMUX_MAX_PSM];
 
-  GstSegment	  sink_segment;
-  GstSegment	  src_segment;
+  GstSegment     sink_segment;
+  GstSegment     src_segment;
 
   /* stream output */
-  GstFluPSStream *current_stream;
-  guint64          next_pts;
-  guint64          next_dts;
-  GstFluPSStream **streams;
-
-  /* properties */
-  GstFluPSDemuxSync sync_method;
-  GstFluPSDemuxSync cur_sync_method;
+  GstFluPSStream * current_stream;
+  guint64        next_pts;
+  guint64        next_dts;
+  GstFluPSStream ** streams;
+  gboolean       need_no_more_pads;
 
   /* Indicates an MPEG-2 stream */
   gboolean is_mpeg2_pack;
 
   /* Language codes event is stored when a dvd-lang-codes
    * custom event arrives from upstream */
-  GstEvent *lang_codes;
+  GstEvent * lang_codes;
 };
 
 struct _GstFluPSDemuxClass {
