@@ -30,6 +30,10 @@
 
 G_BEGIN_DECLS
 
+#if (POST_10_12)
+#define HAVE_LATENCY
+#endif
+
 #define FLUTS_MAX_PID 0x1fff
 
 #define GST_TYPE_FLUTS_DEMUX		(gst_fluts_demux_get_type())
@@ -154,6 +158,12 @@ struct _GstFluTSDemux {
   /* indicates that we need to close our pad group, because we've added
    * at least one pad */
   gboolean need_no_more_pads;
+
+#ifdef HAVE_LATENCY
+  /* clocking */
+  GstClock *clock;
+  GstClockTime clock_base;
+#endif
 };
 
 struct _GstFluTSDemuxClass {
