@@ -31,9 +31,11 @@ GST_DEBUG_CATEGORY (gstflupesfilter_debug);
 static GstFlowReturn gst_pes_filter_data_push (GstPESFilter * filter,
     gboolean first, GstBuffer * buffer);
 
+#define ADAPTER_OFFSET_FLUSH(_bytes_)  if (filter->adapter_offset) *filter->adapter_offset = *filter->adapter_offset + (_bytes_) 
+
 /* May pass null for adapter to have the filter create one */
 void
-gst_pes_filter_init (GstPESFilter * filter, GstAdapter * adapter)
+gst_pes_filter_init (GstPESFilter * filter, GstAdapter * adapter, guint64 * adapter_offset)
 {
   g_return_if_fail (filter != NULL);
 
@@ -43,6 +45,7 @@ gst_pes_filter_init (GstPESFilter * filter, GstAdapter * adapter)
     adapter = gst_adapter_new ();
 
   filter->adapter = adapter;
+  filter->adapter_offset = adapter_offset;
   filter->state = STATE_HEADER_PARSE;
   filter->gather_pes = FALSE;
   filter->allow_unbounded = FALSE;
@@ -56,6 +59,7 @@ gst_pes_filter_uninit (GstPESFilter * filter)
   if (filter->adapter) 
     g_object_unref (filter->adapter);
   filter->adapter = NULL;
+  filter->adapter_offset = NULL;
 }
 
 void
@@ -421,6 +425,7 @@ gst_pes_filter_parse (GstPESFilter * filter)
   }
 
   gst_adapter_flush (filter->adapter, avail);
+  ADAPTER_OFFSET_FLUSH (avail);
 
   return ret;
 
@@ -448,6 +453,7 @@ skip:
   {
     GST_DEBUG ("skipping 0x%02x", filter->id);
     gst_adapter_flush (filter->adapter, avail);
+    ADAPTER_OFFSET_FLUSH (avail);
 
     filter->length -= avail - 6;
     if (filter->length > 0 || filter->unbounded_packet)
@@ -458,6 +464,7 @@ encrypted:
   {
     GST_DEBUG ("skipping encrypted 0x%02x", filter->id);
     gst_adapter_flush (filter->adapter, avail);
+    ADAPTER_OFFSET_FLUSH (avail);
 
     filter->length -= avail - 6;
     if (filter->length > 0 || filter->unbounded_packet)
@@ -468,6 +475,8 @@ lost_sync:
   {
     GST_DEBUG ("lost sync");
     gst_adapter_flush (filter->adapter, 4);
+    ADAPTER_OFFSET_FLUSH (4);
+
     return GST_FLOW_LOST_SYNC;
   }
 }
@@ -548,6 +557,7 @@ gst_pes_filter_process (GstPESFilter * filter)
 
         if (skip) {
           gst_adapter_flush (filter->adapter, avail);
+          ADAPTER_OFFSET_FLUSH (avail);
           ret = GST_FLOW_OK;
         } else {
           GstBuffer *out;
@@ -594,6 +604,8 @@ gst_pes_filter_flush (GstPESFilter * filter)
 
   if (filter->adapter) {
     gst_adapter_clear (filter->adapter);
+    if (filter->adapter_offset)
+      *filter->adapter_offset = G_MAXUINT64;
   }
   filter->state = STATE_HEADER_PARSE;
 }
