@@ -28,9 +28,10 @@ G_BEGIN_DECLS
 
 typedef struct _GstPESFilter GstPESFilter;
 
-typedef GstFlowReturn 	(*GstPESFilterData) 	(GstPESFilter *filter, gboolean first, GstBuffer *buffer, 
-						 gpointer user_data);
-typedef void 		(*GstPESFilterResync) 	(GstPESFilter *filter, gpointer user_data);
+typedef GstFlowReturn (*GstPESFilterData) (GstPESFilter * filter, gboolean first, GstBuffer * buffer, 
+                         gpointer user_data);
+typedef void (*GstPESFilterResync) (GstPESFilter * filter, gpointer user_data);
+typedef void (*GstPESFilterIndex) (GstPESFilter * filter, gpointer user_data);
 
 typedef enum {
   STATE_HEADER_PARSE,
@@ -39,7 +40,8 @@ typedef enum {
 } GstPESFilterState;
 
 struct _GstPESFilter {
-  GstAdapter	    *adapter;
+  GstAdapter         * adapter;
+  guint64            * adapter_offset;
 
   GstPESFilterState  state;
   /* Whether to collect entire PES packets before
@@ -59,26 +61,26 @@ struct _GstPESFilter {
   gboolean           unbounded_packet;
   guint16            length;
 
-  guint8	     type;
+  guint8             type;
 
   gint64             pts;
   gint64             dts;
 };
 
-void		gst_pes_filter_init (GstPESFilter *filter, GstAdapter *adapter);
+void gst_pes_filter_init (GstPESFilter * filter, GstAdapter * adapter, guint64 * adapter_offset);
 
-void		gst_pes_filter_uninit 		(GstPESFilter *filter);
+void gst_pes_filter_uninit (GstPESFilter * filter);
 
-void		gst_pes_filter_set_callbacks 	(GstPESFilter *filter, 
-						 GstPESFilterData data_cb,
-						 GstPESFilterResync resync_cb,
-						 gpointer user_data);
+void gst_pes_filter_set_callbacks (GstPESFilter * filter, 
+                         GstPESFilterData data_cb,
+                         GstPESFilterResync resync_cb,
+                         gpointer user_data);
 
-GstFlowReturn	gst_pes_filter_push		(GstPESFilter *filter, GstBuffer *buffer);
-GstFlowReturn	gst_pes_filter_process		(GstPESFilter *filter);
+GstFlowReturn gst_pes_filter_push (GstPESFilter * filter, GstBuffer * buffer);
+GstFlowReturn gst_pes_filter_process (GstPESFilter * filter);
 
-void		gst_pes_filter_flush		(GstPESFilter *filter);
-GstFlowReturn	gst_pes_filter_drain		(GstPESFilter *filter);
+void gst_pes_filter_flush (GstPESFilter * filter);
+GstFlowReturn gst_pes_filter_drain (GstPESFilter * filter);
 
 G_END_DECLS
 
